/*

this is a part of vattery
copyright 2011 jezra lickter http://www.jezra.net

vattery is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

vattery is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with vattery.	If not, see <http://www.gnu.org/licenses/>.

*/
public enum BatteryDataType {
	PROC,
	SYS
}

public struct BatteryData {
	public int percent;
	public string state;
}
const string DIR_PROC = "/proc/acpi/battery";
const string DIR_SYS = "/sys/class/power_supply";

public class Battery {
	Regex bat_curr_regex_proc;
	Regex bat_tot_regex_proc;
	Regex ac_regex;
	int64 bat_percent;
	string ac_state;
	public BatteryDataType data_type;
	
	
	public Battery() {
		//create a regex to retrieve the current battery power
		try {
			this.bat_curr_regex_proc = new Regex(".*remaining.*:[ \t\n\r\f]*([0-9]*).*",GLib.RegexCompileFlags.CASELESS,RegexMatchFlags.NEWLINE_ANY);
		}catch(GLib.RegexError error) {
			stdout.printf(error.message);
		}
		//create a regex to retrieve the last full capacity of the battery
		try {
			this.bat_tot_regex_proc = new Regex(".*last full capacity:[ \t\n\r\f]*([0-9]*).*",GLib.RegexCompileFlags.CASELESS,RegexMatchFlags.NEWLINE_ANY);
		} catch(GLib.RegexError error) {
			stdout.printf(error.message);
		}
		//create a regex to detect the "charging" status
		try {
			this.ac_regex = new Regex(".*charging.*:[ \t\n\r\f]*([a-z]*).*");
		} catch(GLib.RegexError error) {
			stdout.printf(error.message);
		}
    //check the files and see what sort of data type we should use
		check_data_type();
	}
	
	public BatteryData get_status( ) {
		BatteryData ret = BatteryData();
		//determine if we need to get sys or proc
		switch ( data_type ) {
			case BatteryDataType.SYS : 
				get_sys_status();
				break;
			case BatteryDataType.PROC :
				get_proc_status();
				break;
			default :
				//do nothing?
				break;
		}

		ret.percent = (int)this.bat_percent;
		ret.state = this.ac_state;
		return ret;	
	}
	
	public void check_data_type() {
		string t;
		//can we use the sys?
		if (get_sys_status() ) {
			data_type = BatteryDataType.SYS;
			t="SYS";
		} else if ( get_proc_status() ) {
			data_type = BatteryDataType.PROC;
			t="PROC";
		} else {
			//bummer. unknown batterydatatype
			t="None";
		}
		stdout.printf("using data type: %s\n", t);
	}

	private bool get_proc_status() {
		string contents;
		MatchInfo match_info;
		string bat_curr="";
		string bat_total="";
		string ac_state="";
		int bats_current=0;
		int bats_total=0;
		int bats_percent;
		string state_file;
		string info_file;
		bool battery_data_found=false;
		
		//open the battery root for reading
		bool tloop = true;
		try {
			//stdout.printf("reading %s\n", DIR_PROC);
			GLib.Dir dir = GLib.Dir.open(DIR_PROC,0);
			dir.rewind();
			while (tloop) {
				var name = dir.read_name();
				if(name == null){
					tloop=false;
				}else{
					//something was found
					var check_path = "%s/%s".printf(DIR_PROC,name);
					//is this a directory?
					if(FileUtils.test(check_path,GLib.FileTest.IS_DIR) ) {
						//this is a directory, pretend it is a battery
						state_file = "%s/state".printf(check_path);
						info_file = "%s/info".printf(check_path);
						//does the state_file exist?
						if(FileUtils.test(state_file,FileTest.EXISTS) ){	
							try{
								if ( FileUtils.get_contents(state_file, out contents) ) {
									//regex to get current capacity
									this.bat_curr_regex_proc.match(contents,GLib.RegexMatchFlags.NEWLINE_ANY, out match_info);
									bat_curr = match_info.fetch(1);
									//regex to get the "charging" only if no ac state has been detected so far
									if (ac_state==""){
										this.ac_regex.match(contents,GLib.RegexMatchFlags.NEWLINE_ANY,out match_info);
										ac_state = match_info.fetch(1);
									}
									//now check the info file to get the last recorded max capacity
									if(FileUtils.get_contents(info_file, out contents) ) {
										//regex to get total capacity
										this.bat_tot_regex_proc.match(contents,GLib.RegexMatchFlags.NEWLINE_ANY,out match_info);
										bat_total = match_info.fetch(1);
										//if we ever get this far, then data was found
										battery_data_found=true;
										//add this batteries current charge to the overall charge
										bats_current+= int.parse(bat_curr);
										//add this batteries current total to the overall total
										bats_total+=int.parse(bat_total);
									}
								}
							}catch( GLib.FileError error){
								//there is a chance that no files exist
							}
						}
					}
				}
			}	 
		}catch(Error e){
			stderr.printf( e.message );
		}

		//did we find any data?
		if(battery_data_found){
			bats_percent = bats_current*100/bats_total;
			this.bat_percent = bats_percent;
			this.ac_state = ac_state;
		
		}else{
			this.bat_percent = 0;
			this.ac_state = "NONE";
			stdout.printf("No battery info found in %s\n", DIR_PROC);
			return false;
		}
		return true;
	}
	private bool get_sys_status() {
		string contents;
		string bat_curr="";
		string bat_total="";
		string ac_state="";
		int64 bats_current=0;
		int64 bats_total=0;
		int64 bats_percent;
		string status_file;
		string charge_full_file;
		string charge_now_file;
		bool battery_data_found=false;
		
		//open the battery root for reading
		bool tloop = true;
		try {
			//stdout.printf("Checking %s\n", DIR_SYS);
			GLib.Dir dir = GLib.Dir.open(DIR_SYS,0);
			dir.rewind();
			while (tloop) {
				var name = dir.read_name();
				if(name == null){
					tloop=false;
				}else{
					//something was found, is it a "BAT*"?
					if ( !name.has_prefix("BAT") ) { 
						//nope 
						continue;
					}
					var check_path = "%s/%s".printf(DIR_SYS,name);
					//is this a directory?
					if(FileUtils.test(check_path,GLib.FileTest.IS_DIR) ) {
						//this is a directory, pretend it is a battery
						status_file = "%s/status".printf(check_path);
						//is there a charge_full?
						string file_prefix = "energy";
						bool use_energy = true;
						if (!FileUtils.test("%s/%s_full".printf(check_path,file_prefix), FileTest.EXISTS) ) {
							file_prefix = "charge";
							use_energy = false;
						}
						charge_full_file = "%s/%s_full".printf(check_path, file_prefix);
						charge_now_file = "%s/%s_now".printf(check_path, file_prefix);
						//does the charge_now_file exist?
						if(FileUtils.test(charge_now_file, FileTest.EXISTS) ){	
							try{
								if ( FileUtils.get_contents(charge_now_file, out contents) ) {
									bat_curr = contents;		
									//now check the info file to get the last recorded max capacity
									if(FileUtils.get_contents(charge_full_file, out contents) ) {
										bat_total = contents;
										//if we ever get this far, then data was found
										battery_data_found=true;
										//add this batteries current charge to the overall charge
										bats_current+= int64.parse(bat_curr);
										//add this batteries current total to the overall total
										bats_total+=int64.parse(bat_total);
									}
									
									//now get the charging state
									if(FileUtils.get_contents(status_file, out contents) ) {
										//remove white space and make the content lowercase
										ac_state = contents.strip().down();
									}
								}
							}catch( GLib.FileError error){
								//there is a chance that no files exist
								stderr.printf(error.message+"\n");
							}
						}
					}
				}
			}	 
		}catch(Error e){
			stderr.printf( e.message );
		}

		//did we find any data?
		if(battery_data_found){
			bats_percent = bats_current*100/bats_total;
			this.bat_percent = bats_percent;
			this.ac_state = ac_state;
		
		}else{
			this.bat_percent = 0;
			this.ac_state = "NONE";
			stdout.printf(@"No battery info found in $DIR_SYS\n");
			return false;
		}
		return true;
	}

}
